@@ -6,23 +6,31 @@ import sys
 import os
 import os.path
 from subprocess import PIPE, Popen
+import select
+try:
+    from StringIO import StringIO   # NOQA
+except ImportError:
+    from io import StringIO         # NOQA
 
 from flake8.util import skip_file
 from flake8 import pep8
 from flake8 import pyflakes
 from flake8 import mccabe
 
+pep8style = None
+
 
 def check_file(path, complexity=-1):
     warnings = pyflakes.checkPath(path)
-    warnings += pep8.input_file(path)
+    warnings += pep8style.input_file(path)
     if complexity > -1:
         warnings += mccabe.get_module_complexity(path, complexity)
     return warnings
 
 
 def check_code(code, complexity=-1):
-    warnings = pyflakes.check(code, '<stdin>')
+    warnings = pyflakes.check(code, 'stdin')
+    warnings += pep8style.input_file(StringIO(code))
     if complexity > -1:
         warnings += mccabe.get_code_complexity(code, complexity)
     return warnings
@@ -45,7 +53,9 @@ def _get_python_files(paths):
 
 
 def main():
-    options, args = pep8.process_options()
+    global pep8style
+    pep8style = pep8.StyleGuide(parse_argv=True, config_file=True)
+    options = pep8style.options
     complexity = options.max_complexity
     builtins = set(options.builtins)
     warnings = 0
@@ -53,10 +63,17 @@ def main():
     if builtins:
         orig_builtins = set(pyflakes._MAGIC_GLOBALS)
         pyflakes._MAGIC_GLOBALS = orig_builtins | builtins
-    if args:
-        for path in _get_python_files(args):
+
+    if pep8style.paths and options.filename is not None:
+        for path in _get_python_files(pep8style.paths):
             warnings += check_file(path, complexity)
     else:
+        # wait for 1 second on the stdin fd
+        reads, __, __ = select.select([sys.stdin], [], [], 1.)
+        if reads == []:
+            print('input not specified')
+            raise SystemExit(1)
+
         stdin = sys.stdin.read()
         warnings += check_code(stdin, complexity)
 
@@ -97,6 +114,7 @@ class _PEP8Options(object):
     benchmark = False
     testsuite = ''
     doctest = False
+    max_line_length = pep8.MAX_LINE_LENGTH
 
 
 def _initpep8():
@@ -106,6 +124,7 @@ def _initpep8():
     pep8.options.logical_checks = pep8.find_checks('logical_line')
     pep8.options.counters = dict.fromkeys(pep8.BENCHMARK_KEYS, 0)
     pep8.options.messages = {}
+    pep8.options.max_line_length = 79
     pep8.args = []
 
 
@@ -116,8 +135,11 @@ def run(command):
             [line.strip() for line in p.stderr.readlines()])
 
 
-def git_hook(complexity=-1, strict=False):
+def git_hook(complexity=-1, strict=False, ignore=None):
     _initpep8()
+    if ignore:
+        pep8.options.ignore=ignore
+
     warnings = 0
 
     _, files_modified, _ = run("git diff-index --cached --name-only HEAD")
@@ -150,5 +172,50 @@ def hg_hook(ui, repo, **kwargs):
 
     return 0
 
+
+try:
+    from setuptools import Command
+except ImportError:
+    Flake8Command = None
+else:
+    class Flake8Command(Command):
+        description = "Run flake8 on modules registered in setuptools"
+        user_options = []
+
+        def initialize_options(self):
+            pass
+
+        def finalize_options(self):
+            pass
+
+        def distribution_files(self):
+            if self.distribution.packages:
+                for package in self.distribution.packages:
+                    yield package.replace(".", os.path.sep)
+
+            if self.distribution.py_modules:
+                for filename in self.distribution.py_modules:
+                    yield "%s.py" % filename
+
+        def run(self):
+            global pep8style
+            pep8style = pep8.StyleGuide(config_file=True)
+
+            _initpep8()
+
+            # _get_python_files can produce the same file several
+            # times, if one of its paths is a parent of another. Keep
+            # a set of checked files to de-duplicate.
+            checked = set()
+
+            warnings = 0
+            for path in _get_python_files(self.distribution_files()):
+                if path not in checked:
+                    warnings += check_file(path)
+                checked.add(path)
+
+            raise SystemExit(warnings > 0)
+
+
 if __name__ == '__main__':
     main()
